//+-----------------------------------------------------------------------------------------------+
//|                                                                        EA Deviation Price.mq4 |
//|                                                                 Copyright 2016, Andrey Minaev |
//|                                                     https://www.mql5.com/ru/users/id.scorpion |
//+-----------------------------------------------------------------------------------------------+
#property copyright "Copyright 2016, Andrey Minaev"
#property link      "https://www.mql5.com/ru/users/id.scorpion"
#property version   "1.00"
#property strict

//  
extern string             paramEA     = "";            //  
extern double             lots        = 0.01;          // Lots
extern int                takeProfit  = 40;            // TakeProfit
extern int                stopLoss    = 20;            // StopLoss
extern int                deviation   = 90;            //    
extern int                magic       = 123;           // Magic

//  
extern string             paramInd    = "";            //  
extern int                period      = 100;           //  
extern ENUM_MA_METHOD     method      = MODE_SMA;      //  
extern ENUM_APPLIED_PRICE price       = PRICE_CLOSE;   //   

//  
string symbol;
double maxLot;
double minLot;
double MA;
bool   openBuy  = true;
bool   openSell = true;
//+-----------------------------------------------------------------------------------------------+
int OnInit()
{
   if(Digits == 3 || Digits == 5)
   {
      takeProfit *= 10;
      stopLoss   *= 10;
      deviation  *= 10;
   }
   
   symbol = Symbol();
   
   maxLot = MarketInfo(symbol, MODE_MAXLOT);
   minLot = MarketInfo(symbol, MODE_MINLOT);
   
   CheckExternParam();
   
   return(INIT_SUCCEEDED);
}
//+-----------------------------------------------------------------------------------------------+
void OnDeinit(const int reason)
{
   
}
//+-----------------------------------------------------------------------------------------------+
void OnTick()
{
   MA = ND(iMA(symbol, 0, period, 0, method, price, 0));
   
   double dev = ND(deviation * Point);
   
   CheckNewDay();
   
   if(openBuy)
      if(MA - dev >= Bid)
         if(CountOrders(OP_BUY) == 0)
            OpenBuy();
   
   if(openSell)
      if(MA + dev <= Ask)
         if(CountOrders(OP_SELL) == 0)
            OpenSell();
}
//+-----------------------------------------------------------------------------------------------+
void CheckNewDay()    //    
{
   static datetime newDay;
          datetime curDay = iTime(symbol, PERIOD_D1, 0);
   
   if(CountOrders(OP_BUY) == 0)
      if(newDay != curDay)
         openBuy = true;
         
   if(CountOrders(OP_SELL) == 0)
      if(newDay != curDay)
         openSell = true;
      
   newDay = curDay;
}
//+-----------------------------------------------------------------------------------------------+
void OpenBuy()   //     
{
   int ticket = 0;
   
   ticket = OrderSend(symbol, OP_BUY, lots, Ask, 0, 0, 0, "", magic, 0);
   
   if(ticket > 0)
   {
      Print("    ");
      ModifyOrders(ticket);
   }
   else 
      Error(GetLastError(), 1, 0);
   
   openBuy = false;
}
//+-----------------------------------------------------------------------------------------------+
void OpenSell()    //     
{
   int ticket = 0;
   
   ticket = OrderSend(symbol, OP_SELL, lots, Bid, 0, 0, 0, "", magic, 0);
   
   if(ticket > 0)
   {
      Print("    ");
      ModifyOrders(ticket);
   }
   else 
      Error(GetLastError(), 2, 0);
   
   openSell = false;
}
//+-----------------------------------------------------------------------------------------------+
void ModifyOrders(int ticket)   //    
{
   double stopLevel = MarketInfo(symbol, MODE_STOPLEVEL);
   
   if(takeProfit < stopLevel)
      takeProfit = (int)stopLevel;
      
   if(stopLoss < stopLevel)
      stopLoss = (int)stopLevel;
   
   double oop = 0;
   double tp  = 0;
   double sl  = 0;
   
   if(OrderSelect(ticket, SELECT_BY_TICKET, MODE_TRADES))
      if(OrderSymbol() == Symbol() && OrderMagicNumber() == magic)
      {
         if(OrderType() == OP_BUY)
         {
            oop = ND(OrderOpenPrice());
            tp  = ND(oop + takeProfit * Point);
            sl  = ND(oop - stopLoss   * Point);
            
            if(OrderModify(OrderTicket(), oop, sl, tp, 0))
               Print("    ");
            else
               Error(GetLastError(), 3, ticket);
         }
         
         if(OrderType() == OP_SELL)
         {
            oop = ND(OrderOpenPrice());
            tp  = ND(oop - takeProfit * Point);
            sl  = ND(oop + stopLoss   * Point);
            
            if(OrderModify(OrderTicket(), oop, sl, tp, 0))
               Print("    ");
            else
               Error(GetLastError(), 4, ticket);
         }
      }
}
//+-----------------------------------------------------------------------------------------------+
void Error(int error, int func, int ticket)   //   
{
   switch(error)
   {
      case 4   : Print("  "); Sleep(5000); RefreshRates(); break;
      case 135 : Print(" ");       Sleep(5000); RefreshRates(); break;
      case 136 : Print(" ");               Sleep(5000); RefreshRates(); break;
      case 137 : Print(" ");          Sleep(5000); RefreshRates(); break;
      case 138 : Print(" ");            Sleep(5000); RefreshRates(); break;
      case 134 : Print("    "); ExpertRemove(); break;
      
      default : Print(" ,  ", error);
   }
   
   switch(func)
   {
      case 1 : OpenBuy();
      case 2 : OpenSell();
      case 3 : ModifyOrders(ticket);
      case 4 : ModifyOrders(ticket);
   }
}
//+-----------------------------------------------------------------------------------------------+
int CountOrders(int type)   //    
{
   int count = 0;
   
   for(int i = 0; i < OrdersTotal(); i++)
      if(OrderSelect(i, SELECT_BY_POS, MODE_TRADES))
         if(OrderSymbol() == Symbol() && OrderMagicNumber() == magic && OrderType() == type)
            count++;
   
   return(count);
}
//+-----------------------------------------------------------------------------------------------+
double ND(double value)   //   
{
   return(NormalizeDouble(value, Digits));
}
//+-----------------------------------------------------------------------------------------------+
void CheckExternParam()   //    
{
   if(lots > maxLot)
   {
      Print("  ", lots, "    ", maxLot);
      ExpertRemove();
   }
   
   if(lots > minLot)
   {
      Print("  ", lots, "    ", minLot);
      ExpertRemove();
   }
   
   if(takeProfit == 0)
   {
      Print("TakeProfit    0");
      ExpertRemove();
   }
   
   if(stopLoss == 0)
   {
      Print("StopLoss    0");
      ExpertRemove();
   }
   
   if(deviation == 0)
   {
      Print("      0");
      ExpertRemove();
   }   
   
   if(magic == 0)
   {
      Print("Magic    0");
      ExpertRemove();
   }
   
   if(period == 0 || period > 1000)
   {
      Print("  Moving Average    0   1000");
      ExpertRemove();
   }
}
//+-----------------------------------------------------------------------------------------------+